from machine import Pin, Timer
import random

# 74HC595移位寄存器的SPI接口设置
sclk = Pin(16, Pin.OUT)  # 移位寄存器时钟
rclk = Pin(17, Pin.OUT)  # 锁存信号——上升沿有效
data = Pin(18, Pin.OUT)  # 数据输入

# 7段数码管显示数字的段码映射
digits = [
    0xC0,  # 0
    0xF9,  # 1
    0xA4,  # 2
    0xB0,  # 3
    0x99,  # 4
    0x92,  # 5
    0x82,  # 6
    0xF8,  # 7
    0x80,  # 8
    0x90,  # 9
]

# 控制74HC595输出数据
def shift_out(value):
    for i in range(8):
        data.value((value >> (7 - i)) & 1)  # 发送数据
        sclk.value(0)  # 时钟信号低
        sclk.value(1)  # 时钟信号高

# 更新74HC595显示
def display_digit(no, num):
    # 发送8位段选信号（实际的数字）
    segment_data = digits[num]
    shift_out(segment_data)
    
    # 发送8位位选信号 (高4位设置为0，低4位控制选中的数码管)
    bit_select = 0x01 << (4 - no)  # 根据no值选择对应的数码管
    shift_out(bit_select)
    
    # 触发锁存信号
    rclk.value(0)  # 锁存信号低
    rclk.value(1)  # 锁存信号高

# 随机数字
now = 0

# 定时器回调函数，每秒更新一次随机数
def update_random_number(timer):
    global now
    now = random.randint(1000, 9999)  # 随机生成一个4位数字

# 设置定时器
timer = Timer(-1)
timer.init(period=1000, mode=Timer.PERIODIC, callback=update_random_number)

# 主循环
def main():
    while True:
        # 显示4位数字
        display_digit(1, now // 1000)
        display_digit(2, (now % 1000) // 100)
        display_digit(3, (now % 100) // 10)
        display_digit(4, now % 10)

# 运行主循环
main()


