### 本程序的技术博客
### http://blog.mangolovecarrot.net/2025/01/19/pico-03

from machine import Pin, Timer, I2C
import random
import time

# 74HC595移位寄存器的SPI接口设置
sclk = Pin(16, Pin.OUT)  # 移位寄存器时钟
rclk = Pin(17, Pin.OUT)  # 锁存信号——上升沿有效
data = Pin(18, Pin.OUT)  # 数据输入

# 7段数码管显示数字的段码映射（共阳）
digits = [
    0xC0,  # 0
    0xF9,  # 1
    0xA4,  # 2
    0xB0,  # 3
    0x99,  # 4
    0x92,  # 5
    0x82,  # 6
    0xF8,  # 7
    0x80,  # 8
    0x90,  # 9
]
# 横杠和错误的显示
dash = 0xBF  # 4个横杠显示，假设使用的段码 1011 1111 BF
err = [0x86, 0xAF, 0xAF, 0x7F]  # 假设Err用4个数字显示
# PGFE DCBA
# 1000 0110 E
# 1010 1111 r AF 

# 显示模式设置
DISP_MODE_DIGIT = 0
DISP_MODE_DASH = 1
DISP_MODE_ERROR = 2
dispMode = DISP_MODE_DASH  # 0-随机数，1-横杠（产生随机数），2-Err 

# I2C接口设置 (修改为I2C1，SDA=GPIO2, SCL=GPIO3)
i2c = I2C(1, scl=Pin(3), sda=Pin(2), freq=400000)

# 随机数字
new_number = 0

def scan_i2c():
    devices = i2c.scan()
    if len(devices) == 0:
        print("没有检测到I2C设备。")
        return None
    elif len(devices) == 1:
        print(f"检测到1个设备，地址: 0x{devices[0]:02X}")
        return devices[0]
    else:
        print("检测到多个I2C设备:")
        for i, device in enumerate(devices):
            print(f"{i + 1}: 0x{device:02X}")
        while True:
            try:
                choice = int(input("请选择设备 (输入序号): ")) - 1
                if 0 <= choice < len(devices):
                    return devices[choice]
                else:
                    print("无效选择，请重试。")
            except ValueError:
                print("请输入有效数字。")

EEPROM_ADDR = scan_i2c()
if EEPROM_ADDR is None:
    raise Exception("未检测到EEPROM设备，程序终止。")

EEPROM_MEM_ADDR = 0x0000  # 指定写入和读取的地址

# 写入EEPROM数据
def write_eeprom(addr, value):
    addr_high = (addr >> 8) & 0xFF  # 获取高字节地址
    addr_low = addr & 0xFF  # 获取低字节地址
    value_high = (value >> 8) & 0xFF  # 获取高字节的数值
    value_low = value & 0xFF  # 获取低字节的数值
    i2c.writeto(EEPROM_ADDR, bytes([addr_high, addr_low, value_high, value_low]))  # 将地址和数据一起写入
    time.sleep(0.01)

# 读取EEPROM数据
def read_eeprom(addr):
    addr_high = (addr >> 8) & 0xFF  # 获取高字节地址
    addr_low = addr & 0xFF  # 获取低字节地址
    i2c.writeto(EEPROM_ADDR, bytes([addr_high, addr_low]))  # 发送地址
    data = i2c.readfrom(EEPROM_ADDR, 2)  # 读取2字节数据
    return (data[0] << 8) | data[1]  # 将读取的高字节和低字节合并为一个16位数字

tt = 0

# 控制74HC595输出数据
def shift_out(value):
    for i in range(8):
        data.value((value >> (7 - i)) & 1)  # 发送数据
        sclk.value(0)  # 时钟信号低
        sclk.value(1)  # 时钟信号高

# 更新74HC595显示
def display_digit(no, num):
    # 发送8位段选信号（实际的数字）
    segment_data = digits[num]
    shift_out(segment_data)
    
    # 发送8位位选信号 (高4位设置为0，低4位控制选中的数码管)
    bit_select = 0x01 << (4 - no)  # 根据no值选择对应的数码管
    shift_out(bit_select)
    
    # 触发锁存信号
    rclk.value(0)  # 锁存信号低
    rclk.value(1)  # 锁存信号高

# 控制显示横杠
def display_dash():
    for no in range(1,5):
        # 发送8位段选信号
        shift_out(dash)
        # 发送8位位选信号 (高4位设置为0，低4位控制选中的数码管)
        shift_out(0x01 << (4 - no))  # 根据no值选择对应的数码管)
        # 触发锁存信号
        rclk.value(0)  # 锁存信号低
        rclk.value(1)  # 锁存信号高

# 控制显示错误
def display_err():
    for no in range(1,5):
        # 发送8位段选信号
        shift_out(err[no-1])
        # 发送8位位选信号 (高4位设置为0，低4位控制选中的数码管)
        shift_out(0x01 << (4 - no))  # 根据no值选择对应的数码管)
        # 触发锁存信号
        rclk.value(0)  # 锁存信号低
        rclk.value(1)  # 锁存信号高

def time_tick_1ms(timer):
    global tt
    global dispMode
    global new_number
    
    tt = tt + 1
    
    if dispMode == DISP_MODE_DIGIT or dispMode == DISP_MODE_ERROR:
        if tt > 1000:
            tt = 0
            # 随机生成一个新的4位数字
            new_number = random.randint(1000, 9999)
            # 写入这个数字
            write_eeprom(EEPROM_MEM_ADDR, new_number)
            # 切换成显示横线
            dispMode = DISP_MODE_DASH
            
            print(f"write_number={new_number}")
    elif dispMode == DISP_MODE_DASH:
        if tt > 500:
            tt = 0
            # 读取EEPROM中的数据并验证
            read_number = read_eeprom(EEPROM_MEM_ADDR)
            if read_number == new_number:
                # 切换成显示数字，表示写入读取成功
                dispMode = DISP_MODE_DIGIT
            else:
                # 切换成显示错误，表示写入的数据和读出的数据不一致
                dispMode = DISP_MODE_ERROR
            
            print(f"read_number ={read_number} -------")

# 设置定时器
timer = Timer(-1)
timer.init(period=1, mode=Timer.PERIODIC, callback=time_tick_1ms)

# 主循环
def main():
    global new_number
    global dispMode

    while True:
        if dispMode == DISP_MODE_DIGIT:
            # 显示4位数字
            display_digit(1, new_number // 1000)
            display_digit(2, (new_number % 1000) // 100)
            display_digit(3, (new_number % 100) // 10)
            display_digit(4, new_number % 10)
        elif dispMode == DISP_MODE_DASH:
            # 显示横杠
            display_dash()
        elif dispMode == DISP_MODE_ERROR:
            # 显示错误信息
            display_err()

# 运行主循环
main()


